/*
 * Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg.
 * All rights reserved.
 * Use of this source code is governed by a MIT-style
 * license that can be found in the LICENSE file.
 */
#include <float.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "parameter.h"
#include "progress.h"
#include "solver.h"
#include "timing.h"

enum VARIANT { SOR = 1, RB, RBA };

int main (int argc, char** argv)
{
    int rank;
    int variant = SOR;

    double S, E;
    Parameter params;
    Solver solver;
    initParameter(&params);

    if (argc < 2) {
        printf("Usage: %s <configFile>\n", argv[0]);
        exit(EXIT_SUCCESS);
    }

    readParameter(&params, argv[1]);
    if (argc == 3) 
    {
        variant = atoi(argv[2]);
    }

    printParameter(&params);
    initSolver(&solver, &params);
#ifndef VERBOSE
    initProgress(solver.te);
#endif

    double tau = solver.tau;
    double te  = solver.te;
    double t   = 0.0;
    int nt     = 0;

    S = getTimeStamp();


    switch (variant) {
    case SOR:
        printf("Plain SOR\n");
        while (t <= te) 
        {
            if (tau > 0.0) computeTimestep(&solver);
            setBoundaryConditions(&solver);
            setSpecialBoundaryCondition(&solver);
            computeFG(&solver);
            computeRHS(&solver);
            if (nt % 100 == 0) normalizePressure(&solver);
            solve(&solver);
            adaptUV(&solver);
            t += solver.dt;
            nt++;

        #ifdef VERBOSE
                printf("TIME %f , TIMESTEP %f\n", t, solver.dt);
        #else
                printProgress(t);
        #endif
        }
        break;
    case RB:
        printf("Red-black SOR\n");
        while (t <= te) 
        {
            if (tau > 0.0) computeTimestep(&solver);
            setBoundaryConditions(&solver);
            setSpecialBoundaryCondition(&solver);
            computeFG(&solver);
            computeRHS(&solver);
            if (nt % 100 == 0) normalizePressure(&solver);
            solveRB(&solver);
            adaptUV(&solver);
            t += solver.dt;
            nt++;

        #ifdef VERBOSE
                printf("TIME %f , TIMESTEP %f\n", t, solver.dt);
        #else
                printProgress(t);
        #endif
        }
        break;
    case RBA:
        printf("Red-black SOR with acceleration\n");
        while (t <= te) 
        {
            if (tau > 0.0) computeTimestep(&solver);
            setBoundaryConditions(&solver);
            setSpecialBoundaryCondition(&solver);
            computeFG(&solver);
            computeRHS(&solver);
            if (nt % 100 == 0) normalizePressure(&solver);
            solveRBA(&solver);
            adaptUV(&solver);
            t += solver.dt;
            nt++;

        #ifdef VERBOSE
                printf("TIME %f , TIMESTEP %f\n", t, solver.dt);
        #else
                printProgress(t);
        #endif
        }
        break;
    }
    
    E = getTimeStamp();
    stopProgress();
    printf("Solution took %.2fs\n", E - S);
    writeResult(&solver);
    return EXIT_SUCCESS;
}
